package com.uas.platform.core.concurrent;


/**
 * 代理runnable来执行
 * 
 * <pre>
 * 涉及到一个runnable对象调用不同参数循环执行的问题，不能直接在IRunnable对象里面设置参数，需要代理执行
 * </pre>
 * 
 * @see Runnable
 * 
 * @author yingp
 * 
 * @param <T>
 *            参数类型
 */
public final class IRunnableAdapter<T> implements Runnable {

	private final IRunnable<T> task;
	private final T param;

	public IRunnableAdapter(IRunnable<T> task, T param) {
		this.task = task;
		this.param = param;
	}

	@Override
	public void run() {
		task.run(param);
	}

}
