package com.uas.platform.core.data.service;

import java.io.Serializable;
import java.util.List;

/**
 * service层的增删改查统一抽象接口
 * 
 * @author yingp
 * 
 * @param <T>
 * @param <ID>
 */
public abstract interface CrudService<T, ID extends Serializable> {

	/**
	 * 按ID查找Entity
	 * 
	 * @param paramID
	 * @return
	 */
	public abstract T findOne(ID paramID);

	/**
	 * 按ID查找Entities
	 * 
	 * @param paramIterable
	 * @return
	 */
	public abstract Iterable<T> findAll(Iterable<ID> paramIterable);

	/**
	 * 保存Entity
	 * 
	 * @param paramS
	 * @return
	 */
	public abstract T save(T paramS);

	/**
	 * 保存Entities
	 * 
	 * @param paramIterable
	 * @return
	 */
	public abstract <S extends T> List<S> save(Iterable<S> paramIterable);

	/**
	 * 按ID删除Entity
	 * 
	 * @param paramID
	 */
	@SuppressWarnings("unchecked")
	public abstract void delete(ID... paramIDs);

	/**
	 * 删除Entity
	 * 
	 * @param paramT
	 */
	public abstract void delete(T paramT);

	/**
	 * 删除Entities
	 * 
	 * @param paramIterable
	 */
	public abstract void delete(Iterable<? extends T> paramIterable);

}
