package com.uas.platform.core.data.support;

import java.lang.reflect.Field;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Column;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.springframework.jdbc.core.ParameterizedPreparedStatementSetter;
import org.springframework.util.StringUtils;

public class Entity<T> {

	private Table table;

	private Map<String, Column> columns;

	private List<Field> fields;
	// 映射模型
	private Class<T> targetCls;
	// 映射关系
	private ParameterizedPreparedStatementSetter<T> rowSetter;
	// ID字段
	private String idProperty;
	// 表字段
	private String insertProperties;

	public Entity(final Class<T> targetCls) {
		this.targetCls = targetCls;
		this.table = targetCls.getAnnotation(Table.class);
		Field[] _fields = this.targetCls.getDeclaredFields();
		this.fields = new ArrayList<Field>();
		this.columns = new HashMap<String, Column>();
		List<String> _insertProperties = new ArrayList<String>();
		for (Field field : _fields) {
			if (!field.isAnnotationPresent(Transient.class) && hasSetterMethod(field, targetCls)) {
				field.setAccessible(true);
				this.fields.add(field);
				String property = field.getName();
				Column column = field.getAnnotation(Column.class);
				if (null != column) {
					this.columns.put(field.getName(), column);
					property = StringUtils.isEmpty(column.name()) ? property : column.name();
				}
				if (field.isAnnotationPresent(Id.class)) {
					idProperty = property;
				}
				_insertProperties.add(property);
			}
		}
		insertProperties = StringUtils.collectionToDelimitedString(_insertProperties, ",");
		this.rowSetter = new ParameterizedPreparedStatementSetter<T>() {

			@Override
			public void setValues(PreparedStatement ps, T argument) throws SQLException {
				int i = 1;
				for (Field field : fields) {
					try {
						Object val = field.get(argument);
						if (null != val) {
							if (field.getType().equals(Date.class)) {
								val = new Timestamp(((Date) val).getTime());
							}
						}
						ps.setObject(i++, val);
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}

		};
	}

	/**
	 * 有set方法
	 * 
	 * @param field
	 * @param targetCls
	 * @return
	 */
	private boolean hasSetterMethod(Field field, Class<?> targetCls) {
		String setMethodName = "set" + StringUtils.capitalize(field.getName());
		try {
			targetCls.getMethod(setMethodName, field.getType());
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	public Table getTable() {
		return table;
	}

	public void setTable(Table table) {
		this.table = table;
	}

	public Map<String, Column> getColumns() {
		return columns;
	}

	public void setColumns(Map<String, Column> columns) {
		this.columns = columns;
	}

	public Class<T> getTargetCls() {
		return targetCls;
	}

	public void setTargetCls(Class<T> targetCls) {
		this.targetCls = targetCls;
	}

	public String getTableName() {
		return null == table ? this.targetCls.getSimpleName() : table.name();
	}

	public List<Field> getFields() {
		return fields;
	}

	public void setFields(List<Field> fields) {
		this.fields = fields;
	}

	public ParameterizedPreparedStatementSetter<T> getRowSetter() {
		return rowSetter;
	}

	public void setRowSetter(ParameterizedPreparedStatementSetter<T> rowSetter) {
		this.rowSetter = rowSetter;
	}

	public String getIdProperty() {
		return idProperty;
	}

	public void setIdProperty(String idProperty) {
		this.idProperty = idProperty;
	}

	public String getInsertProperties() {
		return insertProperties;
	}

	public void setInsertProperties(String insertProperties) {
		this.insertProperties = insertProperties;
	}

}
