package com.uas.platform.core.data.support;

import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ParameterizedPreparedStatementSetter;

import com.uas.platform.core.util.StringUtil;

public class JdbcUtil {

	private static final EntityFactory factory = EntityFactory.getFactory();

	public static <T> Entity<T> getEntity(Class<T> targetCls) {
		return factory.getEntity(targetCls);
	}

	public static <T> String getTableName(Class<T> targetCls) {
		return getEntity(targetCls).getTableName();
	}

	public static <T> ParameterizedPreparedStatementSetter<T> getRowSetter(Class<T> targetCls) {
		return getEntity(targetCls).getRowSetter();
	}
	
	/**
	  * 创建时间 :2017年1月10日 下午5:19:39
	  * @author yujia
	  * @param targets
	  * @param targetCls
	  * @param jdbcTemplate
	  * @return void
	  * @throws
	  * @Description: 批量新增
	  * 
	  */
	public static <T> void batchInsert(List<T> targets, Class<T> targetCls, JdbcTemplate jdbcTemplate) {
		Entity<T> entity = getEntity(targetCls);
		StringBuffer sql = new StringBuffer("insert into ").append(entity.getTableName()).append("(")
				.append(entity.getInsertProperties()).append(") values (").append(StringUtil.repeat("?", entity.getFields().size(), ",")).append(")");
		jdbcTemplate.batchUpdate(sql.toString(), targets, targets.size(), getRowSetter(targetCls));
	}

	public static Object getResultSetValue(ResultSet rs, String field, Class<?> requiredType) throws SQLException {
		if (String.class.equals(requiredType)) {
			return rs.getString(field);
		} else if (boolean.class.equals(requiredType) || Boolean.class.equals(requiredType)) {
			return rs.getBoolean(field);
		} else if (byte.class.equals(requiredType) || Byte.class.equals(requiredType)) {
			return rs.getByte(field);
		} else if (short.class.equals(requiredType) || Short.class.equals(requiredType)) {
			return rs.getShort(field);
		} else if (int.class.equals(requiredType) || Integer.class.equals(requiredType)) {
			return rs.getInt(field);
		} else if (long.class.equals(requiredType) || Long.class.equals(requiredType)) {
			return rs.getLong(field);
		} else if (float.class.equals(requiredType) || Float.class.equals(requiredType)) {
			return rs.getFloat(field);
		} else if (double.class.equals(requiredType) || Double.class.equals(requiredType) || Number.class.equals(requiredType)) {
			return rs.getDouble(field);
		} else if (BigDecimal.class.equals(requiredType)) {
			return rs.getBigDecimal(field);
		} else if (java.sql.Date.class.equals(requiredType)) {
			return rs.getDate(field);
		} else if (java.sql.Time.class.equals(requiredType)) {
			return rs.getTime(field);
		} else if (java.sql.Timestamp.class.equals(requiredType) || java.util.Date.class.equals(requiredType)) {
			return rs.getTimestamp(field);
		} else if (byte[].class.equals(requiredType)) {
			return rs.getBytes(field);
		} else if (Blob.class.equals(requiredType)) {
			return rs.getBlob(field);
		} else if (Clob.class.equals(requiredType)) {
			return rs.getClob(field);
		}
		return rs.getObject(field);
	}

}
