package com.uas.platform.core.data.support;

import java.lang.reflect.Field;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;

import com.uas.platform.core.persistence.EnterpriseUU;
import com.uas.platform.core.persistence.StatusColumn;
import com.uas.platform.core.persistence.UserUU;
import com.uas.platform.core.persistence.Logger;

public class LogUtils {

	/**
	 * 基于注解获取记录所需的参数
	 * 
	 * @param cls 实体类
	 * @return
	 */
	public static LoggerParams parseAnnotation(Class<?> cls) {
		if (cls.isAnnotationPresent(Entity.class) &&
				cls.isAnnotationPresent(Table.class) &&
				cls.isAnnotationPresent(Logger.class)) {
			LoggerParams params = new LoggerParams();
			// Logger annotation
			Logger logger = cls.getAnnotation(Logger.class);
			params.setEnUUColumnName(logger.enUU());
			params.setEnUUColumn(logger.enUU());
			params.setUserUUColumnName(logger.userUU());
			params.setUserUUColumn(logger.userUU());
			// table annotation
			Table table = cls.getAnnotation(Table.class);
			params.setTableName(table.name());
			// id annotation
			Field[] fields = cls.getDeclaredFields();
			for (Field field : fields) {
				if (field.isAnnotationPresent(Id.class)) {
					params.setIdColumn(field.getName());
					if (field.isAnnotationPresent(Column.class)) {
						Column column = field.getAnnotation(Column.class);
						params.setIdColumnName(column.name());
					} else {
						params.setIdColumnName(field.getName());
					}
				} else if (field.isAnnotationPresent(EnterpriseUU.class)) {
					params.setEnUUColumn(field.getName());
					if (field.isAnnotationPresent(Column.class)) {
						Column column = field.getAnnotation(Column.class);
						params.setEnUUColumnName(column.name());
					} else {
						params.setEnUUColumnName(field.getName());
					}
				} else if (field.isAnnotationPresent(UserUU.class)) {
					params.setUserUUColumn(field.getName());
					if (field.isAnnotationPresent(Column.class)) {
						Column column = field.getAnnotation(Column.class);
						params.setUserUUColumnName(column.name());
					} else {
						params.setUserUUColumnName(field.getName());
					}
				} else if (field.isAnnotationPresent(StatusColumn.class)) {
					params.setStatusAnnotation(field.getAnnotation(StatusColumn.class));
					params.setStatusColumn(field.getName());
					if (field.isAnnotationPresent(Column.class)) {
						Column column = field.getAnnotation(Column.class);
						params.setStatusColumnName(column.name());
					} else {
						params.setStatusColumnName(field.getName());
					}
				} else if (field.isAnnotationPresent(Column.class)) {
					Column column = field.getAnnotation(Column.class);
					if (logger.enUU().equals(column.name())) {
						params.setEnUUColumn(field.getName());
					} else if (logger.userUU().equals(column.name())) {
						params.setUserUUColumn(field.getName());
					}
				}
			}
			return params;
		}
		return null;
	}

}
