package com.uas.platform.core.model;

public enum ReleaseStatus {
	/**
	 * {@code 101已发布}
	 */
	released(101, "已发布"),

	/**
	 * {@code 102未发布}
	 */
	unreleased(102, "未发布"),

	/**
	 * {@code 103品牌不存在}
	 */
	error_brand(103, "品牌不存在"),
	/**
	 * {@code 103器件不存在}
	 */
	error_component(104, "器件不存在"),
	
	/**
	 * {@code 105 生产日期为空}
	 */
	error_productDate(105, "生产日期为空"),
	
	/**
	 * {@code 106库存类型为空}
	 */
	error_origial(106, "库存类型为空"),
	
	/**
	 * {@code 107 库存数量为空}
	 */
	error_reserve(107, "库存数量为空"),
	
	/**
	 * {@code 108  最小包单价都为空}
	 */
	error_minPackPrice(108, "最小包单价都为空"),
	
	/**
	 * {@code 109  交期都为空}
	 */
	error_deliveryTime(109, "交期都为空"),
	
	/**
	 * {@code 110 价格信息不统一}
	 */
	error_priceNUnite(110, "价格信息不统一"),
	
	/**
	 * {@code 1101 分段价格错误}
	 */
	error_sectionPrice(111, "分段价格错误"),

	/**
	 * {@code 112 匹配成功}
	 */
	success(112, "匹配成功"),
	
	/**
	 * {@code 113  匹配失败}
	 */
	failure(113, "匹配失败");
	/**
	 * @param value
	 *            发布状态
	 *            <p>
	 *            101<i>已发布</i>
	 *            </p>
	 *            <p>
	 *            102<i>未发布</i>
	 *            </p>
	 *            <p>
	 *            103<i>品牌不存在</i>
	 *            </p>
	 *            <p>
	 *            104<i>器件不存在</i>
	 *            </p>
	 *            <p>
	 *            105<i>匹配成功</i>
	 *            </p>
	 *
	 */
	private ReleaseStatus(int value, String phrase) {
		this.value = value;
		this.phrase = phrase;
	}

	private final int value;

	private final String phrase;

	/**
	 * @return 发布状态的编码
	 */
	public int value() {
		return this.value;
	}

	/**
	 * @return 业务动作描述
	 */
	public String getPhrase() {
		return this.phrase;
	}

	/**
	 * @param statusCode
	 *            状态的编码
	 * @return 状态
	 * @throws IllegalArgumentException
	 *             如果statusCode不存在的话
	 */
	public static ReleaseStatus valueOf(int statusCode) {
		for (ReleaseStatus status : values()) {
			if (status.value == statusCode) {
				return status;
			}
		}
		throw new IllegalArgumentException("没有与编号 [" + statusCode + "]匹配的业务状态");
	}

	/**
	 * 返回状态的编号
	 */
	@Override
	public String toString() {
		return Integer.toString(value);
	}
}
