package com.uas.platform.core.model;

public enum ServiceCode {
	/**
	 * {@code 101器件申请}
	 */
	componentApply(101, "器件申请"),

	/**
	 * {@code 102品牌申请}
	 */
	brandApply(102, "品牌申请"),

	/**
	 * {@code 103器件审批}
	 */
	componentAudit(103, "器件审核"),

	/**
	 * {@code 104品牌审批}
	 */
	brandAudit(104, "品牌审核"),

	/**
	 * {@code 201 未下载}
	 */
	not_Download(201, "未下载"),

	/**
	 * {@code 202 已下载}
	 */
	download(202, "已下载");

	/**
	 * @param value
	 *            业务动作编码
	 *            <p>
	 *            101<i>器件申请</i>
	 *            </p>
	 *            <p>
	 *            102<i>品牌申请</i>
	 *            </p>
	 *            <p>
	 *            103<i>器件审批</i>
	 *            </p>
	 *            <p>
	 *            104<i>品牌审批</i>
	 *            </p>
	 *            <p>
	 *            201<i>未下载</i>
	 *            </p>
	 *            <p>
	 *            202<i>已下载</i>
	 *            </p>
	 */
	private ServiceCode(int value, String phrase) {
		this.value = value;
		this.phrase = phrase;
	}

	private final int value;

	private final String phrase;

	/**
	 * @return 业务动作的编码
	 */
	public int value() {
		return this.value;
	}

	/**
	 * @return 业务动作描述
	 */
	public String getPhrase() {
		return this.phrase;
	}

	/**
	 * @param statusCode
	 *            状态的编码
	 * @return 状态
	 * @throws IllegalArgumentException
	 *             如果statusCode不存在的话
	 */
	public static ServiceCode valueOf(int statusCode) {
		for (ServiceCode status : values()) {
			if (status.value == statusCode) {
				return status;
			}
		}
		throw new IllegalArgumentException("没有与编号 [" + statusCode + "]匹配的业务动作");
	}

	/**
	 * 返回状态的编号
	 */
	@Override
	public String toString() {
		return Integer.toString(value);
	}
}
