package com.uas.platform.core.util;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;

public class FileUtils {

	static final int buffer_size = 1024 * 10;
	static final byte[] buffer = new byte[buffer_size];

	/**
	 * 文件下载
	 * 
	 * @param file
	 *            文件
	 * @param response
	 *            响应请求
	 */
	public static void download(File file, HttpServletResponse response) throws IOException {
		InputStream in = new FileInputStream(file);
		OutputStream os = response.getOutputStream();
		response.addHeader("Content-Disposition", "attachment;");
		response.addHeader("Content-Length", String.valueOf(file.length()));
		response.setCharacterEncoding("utf-8");
		response.setContentType("application/octec-stream");
		int data = 0;
		while ((data = in.read(buffer)) != -1) {
			os.write(buffer, 0, data);
		}
		in.close();
		os.close();
	}

	/**
	 * 文件下载
	 * 
	 * @param content
	 *            内容
	 * @param fileName
	 *            文件名
	 * @param response
	 *            响应请求
	 * @throws IOException
	 */
	public static void download(String content, String fileName, HttpServletResponse response) throws IOException {
		byte[] bytes = content.getBytes();
		OutputStream os = response.getOutputStream();
		String charset = "utf-8";
		response.addHeader("Content-Disposition", "attachment;filename=" + new String(fileName.getBytes(charset), "iso-8859-1"));
		response.addHeader("Content-Length", String.valueOf(bytes.length));
		response.setCharacterEncoding(charset);
		response.setContentType("application/octec-stream");
		os.write(bytes);
		os.close();
	}

	/**
	 * 文件下载，显示进度
	 * 
	 * @param file
	 *            文件
	 * @param response
	 *            响应请求
	 * @param monitor
	 *            监听进度
	 */
	public static void download(File file, HttpServletResponse response, Monitor<Integer> monitor) throws IOException {
		long fileSize = file.length();
		InputStream in = new FileInputStream(file);
		OutputStream os = response.getOutputStream();
		response.addHeader("Content-Disposition", "attachment;");
		response.addHeader("Content-Length", String.valueOf(fileSize));
		response.setCharacterEncoding("utf-8");
		response.setContentType("application/octec-stream");
		int data = 0;
		long downloadData = 0;
		int process = 0;
		int current = 0;
		while ((data = in.read(buffer)) != -1) {
			os.write(buffer, 0, data);
			downloadData += data;
			current = (int) Math.ceil(downloadData * 100 / fileSize);
			if (current > process) {
				process = current;
				monitor.run(process);
			}
		}
		in.close();
		os.close();
	}

	/**
	 * 压缩文件
	 * 
	 * @param srcFiles
	 *            待压缩文件
	 * @param zipFile
	 *            压缩后保存的文件
	 * @return
	 */
	public static boolean zip(String[] srcFiles, String zipFile) {
		String[] fileNames = new String[srcFiles.length - 1];
		for (int i = 0; i < srcFiles.length - 1; i++) {
			fileNames[i] = getFileName(srcFiles[i]);
		}
		BufferedOutputStream bos = null;
		ZipOutputStream zos = null;
		try {
			bos = new BufferedOutputStream(new FileOutputStream(zipFile));
			zos = new ZipOutputStream(bos);
			String entryName = null;
			for (int i = 0; i < fileNames.length; i++) {
				entryName = fileNames[i];
				ZipEntry entry = new ZipEntry(entryName);
				zos.putNextEntry(entry);
				BufferedInputStream bis = new BufferedInputStream(new FileInputStream(srcFiles[i]));
				while (bis.read(buffer, 0, buffer_size) != -1) {
					zos.write(buffer, 0, buffer_size);
				}
				bis.close();
				zos.closeEntry();
			}
			zos.flush();
			zos.close();
			return true;
		} catch (IOException e) {
		} finally {
			IOUtils.closeQuietly(zos, bos);
		}
		return false;
	}

	/**
	 * 压缩文件
	 * 
	 * @param srcDir
	 *            待压缩文件夹
	 * @return
	 */
	public static File zip(String srcDir) {
		return zip(srcDir, null);
	}

	/**
	 * 压缩文件
	 * 
	 * @param srcDir
	 *            待压缩文件夹
	 * @param zipFile
	 *            压缩后保存的文件
	 * @return
	 */
	public static File zip(String srcDir, String zipFile) {
		File zip = null;
		File source = new File(srcDir);
		if (source.exists()) {
			// 默认压缩文件名: 源文件名.zip
			if (zipFile == null)
				zip = new File(source.getParent(), source.getName() + ".zip");
			else
				zip = new File(zipFile);
			if (zip.exists())
				zip.delete(); // 删除旧的文件
			FileOutputStream fos = null;
			ZipOutputStream zos = null;
			try {
				fos = new FileOutputStream(zip);
				zos = new ZipOutputStream(new BufferedOutputStream(fos));
				// 添加对应的文件Entry
				if (source.isDirectory()) {
					for (File file : source.listFiles()) {
						addEntry("", file, zos);
					}
				} else
					addEntry("", source, zos);
			} catch (IOException e) {
				throw new RuntimeException(e);
			} finally {
				IOUtils.closeQuietly(zos, fos);
			}
		}
		return zip;
	}

	/**
	 * 扫描添加文件Entry
	 * 
	 * @param base
	 *            基路径
	 * 
	 * @param source
	 *            源文件
	 * @param zos
	 *            Zip文件输出流
	 * @throws IOException
	 */
	private static void addEntry(String base, File source, ZipOutputStream zos) throws IOException {
		String entry = base + source.getName();
		if (source.isDirectory()) {
			for (File file : source.listFiles()) {
				// 递归列出目录下的所有文件
				addEntry(entry + File.separator, file, zos);
			}
		} else {
			FileInputStream fis = null;
			BufferedInputStream bis = null;
			try {
				fis = new FileInputStream(source);
				bis = new BufferedInputStream(fis, buffer_size);
				int read = 0;
				zos.putNextEntry(new ZipEntry(entry));
				while ((read = bis.read(buffer, 0, buffer_size)) != -1) {
					zos.write(buffer, 0, read);
				}
				zos.closeEntry();
			} finally {
				IOUtils.closeQuietly(bis, fis);
			}
		}
	}

	/**
	 * 解析文件名
	 * 
	 * @param filePath
	 * @return
	 */
	public static String getFileName(String filePath) {
		int location = filePath.lastIndexOf("/");
		String fileName = filePath.substring(location + 1);
		return fileName;
	}

	/**
	 * 解压文件
	 * 
	 * @param zipFile
	 *            压缩文件
	 * @param parentDir
	 *            解压到的路径
	 * @return
	 */
	public static List<String> unzip(String zipFile, String parentDir) {
		try {
			return unzip(new FileInputStream(zipFile), parentDir);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 解压文件
	 * 
	 * @param inputStream
	 *            压缩文件流
	 * @param parentDir
	 *            解压到的路径
	 * @return
	 */
	public static List<String> unzip(InputStream inputStream, String parentDir) {
		try {
			List<String> entries = new ArrayList<String>();
			BufferedInputStream bis = new BufferedInputStream(inputStream);
			ZipInputStream zis = new ZipInputStream(bis);
			BufferedOutputStream bos = null;
			ZipEntry entry = null;
			while ((entry = zis.getNextEntry()) != null) {
				String entryName = entry.getName();
				if (entry.isDirectory()) {
					new File(parentDir, entryName).mkdir();
				} else {
					bos = new BufferedOutputStream(openOutputStream(new File(parentDir, entryName), false));
					int b = 0;
					while ((b = zis.read(buffer, 0, buffer_size)) != -1) {
						bos.write(buffer, 0, b);
					}
					bos.flush();
					bos.close();
					entries.add(entryName);
					zis.closeEntry();
				}
			}
			zis.close();
			return entries;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Opens a {@link FileOutputStream} for the specified file, checking and creating the parent directory if it does not exist.
	 * <p>
	 * At the end of the method either the stream will be successfully opened, or an exception will have been thrown.
	 * <p>
	 * The parent directory will be created if it does not exist. The file will be created if it does not exist. An exception is thrown if the file object exists but is a directory. An exception is thrown if the file exists but cannot be written to. An exception is thrown if the parent directory cannot be created.
	 * 
	 * @param file
	 *            the file to open for output, must not be <code>null</code>
	 * @param append
	 *            if <code>true</code>, then bytes will be added to the end of the file rather than overwriting
	 * @return a new {@link FileOutputStream} for the specified file
	 * @throws IOException
	 *             if the file object is a directory
	 * @throws IOException
	 *             if the file cannot be written to
	 * @throws IOException
	 *             if a parent directory needs creating but that fails
	 */
	public static FileOutputStream openOutputStream(File file, boolean append) throws IOException {
		if (file.exists()) {
			if (file.isDirectory()) {
				throw new IOException("File '" + file + "' exists but is a directory");
			}
			if (file.canWrite() == false) {
				throw new IOException("File '" + file + "' cannot be written to");
			}
		} else {
			File parent = file.getParentFile();
			if (parent != null) {
				if (!parent.mkdirs() && !parent.isDirectory()) {
					throw new IOException("Directory '" + parent + "' could not be created");
				}
			}
		}
		return new FileOutputStream(file, append);
	}

	/**
	 * 图片尺寸
	 * 
	 * @param is
	 * @return
	 */
	public static String getImagePixel(InputStream is) {
		try {
			BufferedImage buffer = ImageIO.read(is);
			return buffer.getWidth() + "x" + buffer.getHeight();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 图片尺寸
	 * 
	 * @param is
	 * @return
	 */
	public static String getImagePixel(File file) {
		InputStream io = null;
		try {
			io = new FileInputStream(file);
			return getImagePixel(io);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} finally {
			IOUtils.closeQuietly(io);
		}
	}

	/**
	 * 图片尺寸
	 * 
	 * @param is
	 * @return
	 */
	public static String getImagePixel(byte[] bytes) {
		InputStream io = null;
		try {
			io = new ByteArrayInputStream(bytes);
			return getImagePixel(io);
		} finally {
			IOUtils.closeQuietly(io);
		}
	}

}