package com.uas.platform.core.util;

import java.math.BigDecimal;
import java.text.DecimalFormat;

/**
 * 数值处理工具
 * 
 * @author yingp
 * 
 */
public class NumberUtil {

	/**
	 * format double类型
	 * 
	 * @param number
	 *            需要format的数据
	 * @param f
	 *            保留f位的小数
	 * @return format之后的double类型的数据
	 */
	public static double formatDouble(double number, int f) {
		if (f > 0) {
			BigDecimal b = new BigDecimal(Double.toString(number));
			BigDecimal one = new BigDecimal("1");
			return b.divide(one, f, BigDecimal.ROUND_HALF_UP).doubleValue();
		} else {
			return Math.floor(number);
		}
	}

	/**
	 * format double类型
	 * 
	 * @param number
	 *            需要format的数据
	 * @param f
	 *            保留f位的小数
	 * @return format之后的double类型的数据
	 */
	public static double formatDouble(String number, int f) {
		double n = Double.parseDouble(number);
		if (f > 0) {
			BigDecimal b = new BigDecimal(number);
			BigDecimal one = new BigDecimal("1");
			return b.divide(one, f, BigDecimal.ROUND_HALF_UP).doubleValue();
		} else {
			return Math.floor(n);
		}
	}

	/**
	 * 浮点型转成BigDecimal
	 * 
	 * @param number
	 * @return
	 */
	public static String parseBigDecimal(double number) {
		return String.valueOf(BigDecimal.valueOf(number));
	}

	/**
	 * 数字格式化
	 * 
	 * @param number
	 * @param f
	 * @return
	 */
	public static String formatNumber(Object number, int f) {
		if ("0".equals(String.valueOf(number)))
			return "0";
		DecimalFormat df = (DecimalFormat) DecimalFormat.getInstance();
		df.setGroupingSize(3);
		int len = number.toString().length();
		if (number.toString().indexOf(".") > 0)
			len = number.toString().indexOf(".");
		String pattern = len > 3 ? "0,000" : "0";
		for (int i = 0; i < f; i++) {
			if (i == 0)
				pattern += ".";
			pattern += "0";
		}
		df.applyPattern(pattern);
		try {
			return df.format(number);
		} catch (Exception e) {
			return null;
		}
	}

	/**
	 * 设置小数点保留位数
	 * */
	public static float subFloat(float f, int length) {
		String fStr = String.valueOf(f);
		int i = fStr.indexOf(".");
		String returnStr = null;
		if (fStr.length() > i + 1 + length) {
			returnStr = fStr.substring(0, i + 1 + length);
		} else
			returnStr = fStr;
		float returnf = (Float.valueOf(returnStr)).floatValue();
		return returnf;
	}

	/**
	 * int数组转化成Integer数组
	 */
	public static Integer[] toIntegerArray(int[] arr) {
		int n = arr.length;
		Integer[] iarr = new Integer[n];
		for (int i = 0; i < n; i++) {
			iarr[i] = new Integer(arr[i]);
		}
		return iarr;
	}

	public static Number nvl(Number number, Number ifNullNumber) {
		return number == null ? ifNullNumber : number;
	}

}
