package com.uas.platform.core.util.mq;

import java.util.List;

import com.uas.platform.core.concurrent.Callback;
import com.uas.platform.core.util.ArrayUtils;
import com.uas.platform.core.util.AuthCodeSoap;

/**
 * @author yujia
 * @version 创建时间：2016年1月22日 下午4:04:40 调用营销账户和验证码账户的统一接口
 */
public class SendMessageUtil {

	/**
	 * 2016年1月22日 下午4:10:34
	 * 
	 * @param content
	 *            发送的内容
	 * @param mobile
	 *            需要发送信息的手机号
	 * @param operatorName
	 *            调用当前接口的用户
	 * @param type
	 *            发送的类型 如果是 auth 是调用发送验证码账户的接口，如果是market，则调用的是营销账户的接口
	 * @param callback
	 *            回执
	 * @author yujia 发送信息的接口
	 * @throws Exception
	 */
	public static void sendMessage(String content, List<String> mobile, String operatorName, MessageType type, Callback<MessageLog> callback)
			throws Exception {
		MessageLog log = null;
		if (type == MessageType.auth) {
			log = AuthCodeSoap.Submit(content, ArrayUtils.listToString(mobile, ","), operatorName);
		} else if (type == MessageType.market) {
			log = SmsBase.SendSms(content, ArrayUtils.listToString(mobile, ","), operatorName);
		} else if (type == MessageType.orderpay){//商城收款短信
			log = OrderPaySoap.Submit(content,ArrayUtils.listToString(mobile,","),operatorName);
		}else {
			throw new IllegalArgumentException("传入的type账户不正确，只能是 auth或market");
		}
		log.setSendCount(mobile.size());
		callback.call(log);
	}

	/**
	 * 2016年1月26日 上午9:05:00
	 * 
	 * @param content
	 *            发送的内容
	 * @param mobile
	 *            需要发送信息的手机号
	 * @param operatorName
	 *            调用当前接口的用户
	 * @param type
	 *            发送的类型 如果是 auth 是调用发送验证码账户的接口，如果是market，则调用的是营销账户的接口
	 * @param callback
	 *            回执
	 */
	public static void sendMessage(final String content, final String mobile, final String operatorName, final MessageType type,
			final Callback<MessageLog> callback) {
		new Thread() {
			public void run() {
				MessageLog log = null;
				if (type == MessageType.auth) {
					log = AuthCodeSoap.Submit(content, mobile, operatorName);
				} else if (type == MessageType.market) {
					try {
						log = SmsBase.SendSms(content, mobile, operatorName);
					} catch (Exception e) {
						e.printStackTrace();
					}
				} else {
					throw new IllegalArgumentException("传入的type账户不正确，只能是 auth或market");
				}
				log.setSendCount(1);
				try {
					callback.call(log);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}.start();
	}
}
