package com.uas.platform.core.web.view;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.URLEncoder;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.LocalizedResourceHelper;
import org.springframework.web.servlet.support.RequestContextUtils;
import org.springframework.web.servlet.view.AbstractView;

public abstract class XmlExcelView extends AbstractView {

	private String url;

	private ApplicationContext applicationContext;

	private String fileName;

	public final static int BUFFER_SIZE = 50000;

	private static final String SOURCE_TYPE = ".xml";

	private static final String CONTENT_TYPE = "application/vnd.ms-excel";

	private static final String EXTENSION = ".xls";

	/**
	 * @param url
	 *            模板路径
	 * @param fileName
	 *            导出文件名
	 * @param applicationContext
	 */
	public XmlExcelView(String url, String fileName, ApplicationContext applicationContext) {
		super();
		this.url = url;
		this.fileName = fileName;
		this.applicationContext = applicationContext;
	}

	protected Resource getTemplateSource(HttpServletRequest request) throws Exception {
		LocalizedResourceHelper helper = new LocalizedResourceHelper(applicationContext);
		Locale userLocale = RequestContextUtils.getLocale(request);
		return helper.findLocalizedResource(url, SOURCE_TYPE, userLocale);
	}

	protected void buildExcelDocument(Map<String, Object> map, StringBuffer xmlBuffer, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (url != null) {
			Resource resource = getTemplateSource(request);
			InputStreamReader streamReader = new InputStreamReader(resource.getInputStream());
			BufferedReader reader = null;
			try {
				reader = new BufferedReader(streamReader);
				readXml(reader, xmlBuffer, request, response);
			} finally {
				if (reader != null)
					reader.close();
			}
		}
	}

	protected abstract void readXml(BufferedReader reader, StringBuffer xmlBuffer, HttpServletRequest request, HttpServletResponse response)
			throws Exception;

	@Override
	protected void renderMergedOutputModel(Map<String, Object> map, HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		StringBuffer xmlBuffer = new StringBuffer(BUFFER_SIZE);
		buildExcelDocument(map, xmlBuffer, request, response);
		response.setContentType(CONTENT_TYPE);
		response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "utf-8") + EXTENSION);
		DataOutputStream stream = new DataOutputStream(response.getOutputStream());
		stream.write(xmlBuffer.toString().getBytes());
		stream.flush();
		stream.close();
	}

}
